//-----------------------------------------------------------------------------------------------
//    Copyright (c) 
//
//    Name: ald_Phong_Mapping.fx
//    Author: Gluharev Aleksi Sergeevith (aivle1@yandex.ru)  aka "Aldaron"
//    26.01.2007
//    HTTP://LASTWORLD.NET.RU
//
//    
//-----------------------------------------------------------------------------------------------


float4x4    mat_World_View_Proj; 
float4x4    matWorld;         
float4      View_Pos;        

float4      LightPos;
float       LightFalloff;  

float4      LightColor           = {0.5f, 0.5f, 0.5f, 1.0f};    
 
float       LightSpecular_Itens  = 0.95f;

float       LightAmbient         = 0.5f;    
float       LightSpecular        = 2.5f;    
float       Attenuation          = 1.0f;

float       ParallaxDepth        = 0.04f; 


struct VS_INPUT
{ 
float4 Position   : POSITION;
float3 Normal     : NORMAL;
float2 Tex_UV     : TEXCOORD0;
float3 Tangent    : TANGENT;
float3 Binormal   : BINORMAL;
};

struct VS_OUT
{
float4 Position   : POSITION;
float4 WorldPos   : TEXCOORD0;
float3 ViewNormal : TEXCOORD1;
float2 Tex_UV     : TEXCOORD2;
float3 LightDir   : TEXCOORD4;
float3 ViewDir    : TEXCOORD5;

};

VS_OUT vs_main( in VS_INPUT IN ) 
{
VS_OUT OUT;

    OUT.Position = mul(IN.Position, mat_World_View_Proj);  
    OUT.Tex_UV = IN.Tex_UV; //    

    OUT.WorldPos = mul(IN.Position, matWorld); 
    OUT.LightDir = LightPos - OUT.WorldPos; //  
    
    OUT.ViewDir = View_Pos - OUT.WorldPos; //  
   
    OUT.ViewNormal = normalize( mul( IN.Normal, (float3x3)matWorld) );
    
      
return OUT;
}

texture baseMap;      

sampler baseSampler = sampler_state
{
Texture = (baseMap);
MIPFILTER = ANISOTROPIC;
MAGFILTER = ANISOTROPIC;
MINFILTER = ANISOTROPIC;
};


float4 ps_main( in VS_OUT IN ) : COLOR0
{

float4 Color = tex2D(baseSampler, IN.Tex_UV ); //   

float DotLight = dot( IN.LightDir, IN.LightDir ); // dot product 
IN.LightDir = normalize( IN.LightDir );//     
        
float Atten = min( ((LightFalloff * LightFalloff) / DotLight), Attenuation);// "" (  ) 

IN.ViewDir = normalize(IN.ViewDir);//    

float4 SpecDiff = saturate( dot( normalize( IN.ViewNormal ), IN.LightDir ) ) * Atten;              
float3 Ref      = normalize( 2 * SpecDiff * IN.ViewNormal - IN.LightDir);
float4 Specular = SpecDiff + pow( LightSpecular_Itens * saturate(dot(Ref, IN.ViewDir)), LightSpecular);

return Color =  ( LightAmbient + Specular )  * Color * LightColor * 1.3f; //    
 
}

technique ald_Phong
{
  pass P0
  {
    vertexshader = compile vs_1_1 vs_main();
    pixelshader = compile ps_2_0 ps_main();
  }
}

technique ald_Phong_alpha
{
  pass P0
  {
   AlphaBlendEnable = true;
   SrcBlend         = One;
   DestBlend        = One;
   BlendOp          = Add;
    vertexshader = compile vs_1_1 vs_main();
    pixelshader = compile ps_2_0 ps_main();
  }
}